# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	clients/novell-cifs_proposal.ycp
# Package:	Configuration of novell-cifs
# Summary:	Proposal function dispatcher.
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: novell-cifs_proposal.ycp 27914 2006-02-13 14:32:08Z locilka $
#
# Proposal function dispatcher for novell-cifs configuration.
# See source/installation/proposal/proposal-API.txt
module Yast
  class NovellCifsProposalClient < Client
    def main

      textdomain "novell-cifs"

      Yast.import "NovellCifs"
      Yast.import "Progress"
      Yast.import "NovellLDAP"
      Yast.import "NovellUtils"
      Yast.import "NovellPatterns"
      Yast.import "NovellAddonInstall"

      # The main ()
      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("NovellCifs proposal started")

      @func = Convert.to_string(WFM.Args(0))
      @param = Convert.to_map(WFM.Args(1))
      @ret = {}

      Builtins.y2milestone("NovellCifs proposal function: %1", @func)

      # create a textual proposal
      if @func == "MakeProposal"
        @proposal = ""
        @warning = nil
        @warning_level = nil
        @force_reset = Ops.get_boolean(@param, "force_reset", false)

        if NovellUtils.EdirAskUserFlag("cifs") # If edir changed, then we need to recompute the defaults.
          @force_reset = true
          Builtins.y2milestone(
            "novell-cifs.MakeProposal:force_reset set to true due to ask user flag."
          )
        end

        if @force_reset || !NovellCifs.proposal_valid
          NovellCifs.proposal_valid = true
          @progress_orig = Progress.set(false)
          NovellCifs.Read
          Progress.set(@progress_orig)
        end
        @sum = NovellCifs.Summary
        @proposal = Ops.get_string(@sum, 0, "")
        @ret = NovellAddonInstall.ConstructProposal("novell-cifs", @proposal)
      # validate the current values
      elsif @func == "ValidateValues"
        Builtins.y2milestone("NovellCifs proposal in ValidateValues.")

        @ret = { "errors" => false }

        # Since we are running the config on upgrade, we need to always validate the values.
        #    if( !NovellCifs::upgradeMode )
        #    {
        if !NovellCifs.upgradeMode
          if Ops.get_string(NovellCifs.Settings, "CONFIG_CIFS_LDAP_SERVER", "") == "" ||
              Ops.get_string(
                NovellCifs.Settings,
                "CONFIG_CIFS_SERVER_CONTEXT",
                ""
              ) == "" ||
              Ops.get_string(NovellCifs.Settings, "CONFIG_CIFS_PROXY_USER", "") == "" ||
              Ops.get_string(
                NovellLDAP.Settings,
                "CONFIG_LDAP_ADMIN_PASSWORD",
                ""
              ) == "" ||
              Ops.get_string(
                NovellCifs.Settings,
                "CONFIG_CIFS_PROXY_USER_PASSWORD",
                ""
              ) == ""
            Ops.set(@ret, "errors", true)
          end
        end
      # run the module
      elsif @func == "AskUser"
        Builtins.sleep(750)
        @chosen_id = Ops.get(@param, "chosen_id")
        Builtins.y2milestone("Novell CIFS proposal changing id %1", @chosen_id)

        # Configure now
        if @chosen_id == "novell-cifs--configure_now"
          Ops.set(NovellAddonInstall.products_to_configure, "novell-cifs", true)
          @ret = { "workflow_sequence" => :next }
        elsif @chosen_id == "novell-cifs--configure_later"
          Ops.set(
            NovellAddonInstall.products_to_configure,
            "novell-cifs",
            false
          )
          @ret = { "workflow_sequence" => :next }
        else
          @stored = NovellCifs.ExportSettings
          @seq = Convert.to_symbol(
            WFM.CallFunction("novell-cifs", [path(".propose")])
          )
          NovellCifs.Import(@stored) if @seq != :next
          Builtins.y2debug("stored=%1", @stored)
          Builtins.y2debug("seq=%1", @seq)
          @ret = { "workflow_sequence" => @seq }
        end
      # create titles
      elsif @func == "Description"
        # Determine if we should be in the proposal screen or not.
        @isSelected = NovellPatterns.IsPatternInstalled("novell-cifs")
        Builtins.y2milestone(
          "Proposal:IsSelectionInstalled:novell-cifs: %1",
          @isSelected
        )
        if @isSelected == true
          @ret = {
            # Rich text title for Novell CIFS Services in proposals
            "rich_text_title" => _(
              "OES CIFS Services"
            ),
            # Menu title for Novell CIFS Services in proposals
            "menu_title"      => _(
              "OES &CIFS Services"
            ),
            "id"              => "novell-cifs"
          }
        else
          @ret = {}
        end
      # write the proposal
      elsif @func == "Write"
        NovellCifs.Write
      else
        Builtins.y2error("unknown function: %1", @func)
      end

      # Finish
      Builtins.y2debug("ret=%1", @ret)
      Builtins.y2milestone("NovellCifs proposal finished")
      Builtins.y2milestone("----------------------------------------")
      deep_copy(@ret)

      # EOF
    end
  end
end

Yast::NovellCifsProposalClient.new.main
