# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	include/novell-cifs/dialogs.ycp
# Package:	Configuration of novell-cifs
# Summary:	Dialogs definitions
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: dialogs.ycp 27914 2006-02-13 14:32:08Z locilka $
module Yast
  module NovellCifsDialogsInclude
    def initialize_novell_cifs_dialogs(include_target)
      Yast.import "UI"

      textdomain "novell-cifs"

      Yast.import "Label"
      Yast.import "Wizard"
      Yast.import "Popup"
      Yast.import "Report"
      Yast.import "NovellCifs"
      Yast.import "NovellLDAP"
      Yast.import "NovellLDAPUI"
      Yast.import "NovellUtils"
      Yast.import "NovellValidators"
      Yast.import "NetworkInterfaces"
      Yast.import "NovellLdapBrowser"
      Yast.import "Edirectory"
      Yast.import "EdirUtils"

      Yast.include include_target, "novell-cifs/complex.rb"
      Yast.include include_target, "novell-cifs/helps.rb"

      # globals
      @p_policies = [] # oes password policies
    end

    def PromptIfNeededForEdirAdminPassword
      useXad = false

      if NovellCifs.xadMode
        xad_tree_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          useXad = true
        end # if this value is set at all, then it is the eDir admin (in spite of xad name)
      end
      Builtins.y2milestone(
        "NovellCifs.PromptIfNeededForEdirAdminPassword:use xad:%1",
        useXad
      )

      if useXad
        if Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.XadTreeAdminPasswordDialog
        end
      else
        if Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.AdminPasswordDialog
        end
      end
      Builtins.y2milestone(
        "NovellCifs.PromptIfNeededForEdirAdminPassword:useXad %1",
        useXad
      )
      useXad
    end

    # CifsConfigure1 dialog
    # @return dialog result
    def CifsConfigure1Dialog
      Builtins.y2milestone(
        "NovellCifs.CifsConfigure1Dialog:entering CifsConfigure1Dialog."
      )
      if NovellCifs.upgradeMode
        Popup.Message(NovellUtils.noConfigMessage)
        Builtins.y2milestone(
          "NovellCifs.CifsConfigure1Dialog:skipping GUI since we are in upgrade mode."
        )
        return :skip
      end

      #    if( NovellCifs::upgradeMode )
      #    {
      #	Popup::Message(  NovellUtils::noConfigMessage );
      #        y2milestone("NovellCifs.CifsConfigure1Dialog:skipping GUI since we are in upgrade mode.");
      #        return `skip;
      #    }

      # Novell CIFS LDAP and Secure Channel Configuration
      caption = _("OES CIFS Service Configuration")

      ldap_server = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_LDAP_SERVER",
        ""
      )
      if ldap_server == nil || !NovellLDAP.AddressExists(ldap_server, nil)
        ldap_server = ""
      end
      server_context = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_SERVER_CONTEXT",
        ""
      )
      Builtins.y2milestone(
        "NovellCifs.CifSconfigure1Dialog:server context: %1",
        server_context
      )
      proxy_user = NovellUtils.Nds2Ldap(
        Ops.get_string(NovellCifs.Settings, "CONFIG_CIFS_PROXY_USER", "")
      )
      proxy_user_password = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_PROXY_USER_PASSWORD",
        ""
      )
      use_ocs_for_creds = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_USE_OCS_FOR_CREDS",
        ""
      )
      cifs_ldap_port = Ops.get_integer(
        NovellCifs.Settings,
        "CONFIG_CIFS_LDAP_PORT",
        -1
      )
      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )
      server_list_widget = NovellLDAPUI.ServerListWidget(
        server_list,
        _("eDirectory server address or host name"),
        ldap_server,
        true,
        :combo
      )
      ldap_widget = Ops.get_term(server_list_widget, "term") { VBox(:Empty) }

      reconfigureMode = false
      if Builtins.tolower(NovellCifs.ReadOption("SERVICE_CONFIGURED", "no")) == "yes"
        reconfigureMode = true
      end

      create_new_user = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_CREATE_NEW_PROXY_USER",
        "yes"
      )
      create_new_user = "yes" if create_new_user == nil
      create_new_user = Builtins.tolower(create_new_user)

      create_new_user = "no" if reconfigureMode

      defaultNewProxyUser = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_PROXY_USER_DEFAULT",
        ""
      )
      defaultNewProxyPassword = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_PROXY_USER_DEFAULT_PASSWORD",
        ""
      )
      commonProxyName = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_COMMON_PROXY",
        ""
      )
      commonProxyPassword = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_COMMON_PROXY_PASSWORD",
        ""
      )

      contents = HVSquash(
        VBox(
          ldap_widget,
          VSpacing(1.0),
          TextEntry(
            Id(:ldap_port),
            _("LDAP Port for CIFS Server"),
            Builtins.sformat("%1", cifs_ldap_port)
          ),
          VSpacing(1.0),
          TextEntry(
            Id(:server_context),
            _("Local NCP Server context"),
            NovellUtils.Nds2Ldap(server_context)
          ),
          VSpacing(1.0),
          MultiLineEdit(
            Id(:proxy_user),
            _("Proxy User for CIFS Management"),
            proxy_user
          ),
          VSpacing(1.0),
          HBox(
            Frame(
              _("Credential Storage Location:"),
              MarginBox(
                1.5,
                0.2,
                RadioButtonGroup(
                  Id(:rb),
                  HBox(
                    Left(RadioButton(Id(2), "OCS")),
                    Left(RadioButton(Id(3), _("Local File")))
                  )
                )
              )
            )
          )
        )
      )

      helps = Ops.add(
        Ops.add(
          Ops.get_string(@HELPS, "configure_cifs0", ""),
          Ops.get_string(server_list_widget, "helps", "")
        ),
        Ops.get_string(@HELPS, "configure_cifs1", "")
      )
      Wizard.SetContentsButtons(
        caption,
        contents,
        helps,
        Label.BackButton,
        Label.NextButton
      )

      UI.SetFocus(Id(:next))

      UI.ChangeWidget(Id(:server_context), :Enabled, false) # just there for display purposes
      UI.ChangeWidget(Id(:ldap_port), :Enabled, false) # just there for display purposes
      UI.ChangeWidget(Id(:proxy_user), :Enabled, false)

      if use_ocs_for_creds == "yes"
        UI.ChangeWidget(Id(:rb), :CurrentButton, 2)
      else
        UI.ChangeWidget(Id(:rb), :CurrentButton, 3)
      end

      PromptIfNeededForEdirAdminPassword()

      ret = nil

      while true
        ret = UI.UserInput
        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          ldap_server = Convert.to_string(
            UI.QueryWidget(Id(:server_select), :Value)
          )
          Ops.set(NovellCifs.Settings, "CONFIG_CIFS_LDAP_SERVER", ldap_server)


          Ops.set(
            NovellCifs.Settings,
            "CONFIG_CIFS_CREATE_NEW_PROXY_USER",
            create_new_user
          )

          workProxyName = Convert.to_string(commonProxyName)

          if !NovellValidators.ValidateFDN(workProxyName, false)
            name = workProxyName
            name = NovellUtils.Strip(name, " ")
            if name == nil || name == "" || name == " "
              Popup.Error(_("A valid proxy user must be entered."))
            end
            next
          end

          workProxyPassword = Convert.to_string(commonProxyPassword)

          Ops.set(
            NovellCifs.Settings,
            "CONFIG_CIFS_PROXY_USER_PASSWORD",
            workProxyPassword
          )

          Ops.set(NovellCifs.Settings, "CONFIG_CIFS_PROXY_USER", workProxyName)


          if Convert.to_integer(UI.QueryWidget(Id(:rb), :CurrentButton)) == 2
            Ops.set(
              NovellCifs.Settings,
              "CONFIG_CIFS_USE_OCS_FOR_CREDS",
              "yes"
            )
          else
            Ops.set(NovellCifs.Settings, "CONFIG_CIFS_USE_OCS_FOR_CREDS", "no")
          end

          Ops.set(NovellCifs.Settings, "CONFIG_CIFS_LDAP_PORT", cifs_ldap_port)

          break
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      deep_copy(ret)
    end


    # CifsConfigure2 dialog
    # @return dialog result
    def CifsConfigure2Dialog
      # Novell CIFS LDAP and Secure Channel Configuration
      caption = _("OES CIFS Service Configuration")
      usercontext_rights = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_USERCONTEXT_RIGHTS",
        "yes"
      )
      subtree_search = Ops.get_string(
        NovellCifs.Settings,
        "CONFIG_CIFS_SUBTREE_SEARCH",
        "no"
      )
      id_num = 0
      table_items = []
      Builtins.foreach(
        Ops.get_list(NovellCifs.Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
      ) do |item|
        table_items = Builtins.add(
          table_items,
          Item(Id(id_num), NovellUtils.Ldap2Nds(item))
        )
        id_num = Ops.add(id_num, 1)
      end

      contents = VBox(
        Left(Label(_("eDirectory Contexts:"))),
        HBox(
          Table(
            Id(:gui_table),
            Opt(:notify, :immediate),
            Header(_("eDirectory Contexts")),
            table_items
          ),
          VSpacing(Opt(:vstretch), 5)
        ),
        HBox(
          PushButton(Id(:addBttn), _("&Add")),
          PushButton(Id(:delBttn), _("&Delete"))
        ),
        Left(
          CheckBox(
            Id(:usercontext_rights),
            _("Add Proxy User as Trustee of User Contexts"),
            usercontext_rights == "yes"
          )
        ),
        Left(
          CheckBox(
            Id(:subtree_search),
            _("Enable Subtree Search"),
            subtree_search == "yes"
          )
        )
      )



      helps = Ops.add(
        Ops.get_string(@HELPS, "configure_cifs0", ""),
        Ops.get_string(@HELPS, "configure_cifs2", "")
      )
      Wizard.SetContentsButtons(
        caption,
        contents,
        helps,
        Label.BackButton,
        Label.NextButton
      )

      if NovellLDAP.IsLocalLDAPRunning != 0
        UI.ChangeWidget(Id(:addBttn), :Enabled, false)
      end

      if !(NovellLDAP.IsLocalLDAPRunning == 0 ||
          Ops.get_string(Edirectory.EdirSettings, "CONFIG_EDIR_TREE_TYPE", "") == "existing")
        UI.ChangeWidget(Id(:addBttn), :Enabled, false)
      else
        UI.ChangeWidget(Id(:addBttn), :Enabled, true)
      end

      UI.SetFocus(Id(:next))

      primary_server = NovellLDAP.BrowseButtonLdapServer(nil)
      ldap_server = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
      ldap_sport = Ops.get_integer(
        primary_server,
        "CONFIG_LDAP_SECURE_PORT",
        636
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          usercontext_rights2 = Convert.to_boolean(
            UI.QueryWidget(Id(:usercontext_rights), :Value)
          )
          Ops.set(
            NovellCifs.Settings,
            "CONFIG_CIFS_USERCONTEXT_RIGHTS",
            usercontext_rights2 ? "yes" : "no"
          )
          subtree_search2 = Convert.to_boolean(
            UI.QueryWidget(Id(:subtree_search), :Value)
          )
          Ops.set(
            NovellCifs.Settings,
            "CONFIG_CIFS_SUBTREE_SEARCH",
            subtree_search2 ? "yes" : "no"
          )
          break
        elsif ret == :gui_table
          next
        elsif ret == :addBttn
          newcontext = ""
          contents2 = HVSquash(
            VBox(
              HBox(
                TextEntry(
                  Id(:newcontext),
                  Opt(:notify),
                  _("Add an eDirectory context (e.g. o=microfocus)"),
                  newcontext
                ),
                VBox(Label(""), Left(PushButton(Id(:browse), _("&Browse"))))
              ),
              HBox(
                PushButton(Id(:Ok), Opt(:default), _("&Ok")),
                PushButton(Id(:CancelBttn), _("&Cancel"))
              )
            )
          )
          UI.OpenDialog(Opt(:decorated), contents2)
          UI.SetFocus(Id(:newcontext))

          UI.ChangeWidget(Id(:Ok), :Enabled, false)
          if NovellLDAP.IsLocalLDAPRunning != 0
            UI.ChangeWidget(Id(:browse), :Enabled, false)
          end

          if !(NovellLDAP.IsLocalLDAPRunning == 0 ||
              Ops.get_string(
                Edirectory.EdirSettings,
                "CONFIG_EDIR_TREE_TYPE",
                ""
              ) == "existing")
            UI.ChangeWidget(Id(:browse), :Enabled, false)
          else
            UI.ChangeWidget(Id(:browse), :Enabled, true)
          end

          while true
            ret2 = UI.UserInput
            if ret2 == :cancel || ret2 == :CancelBttn
              UI.CloseDialog
              break
            elsif ret2 == :browse
              adminDn = Ops.get_string(
                NovellLDAP.Settings,
                "CONFIG_LDAP_ADMIN_CONTEXT",
                ""
              )
              adminPassword = Ops.get_string(
                NovellLDAP.Settings,
                "CONFIG_LDAP_ADMIN_PASSWORD",
                ""
              )
              if NovellCifs.xadMode
                xad_tree_admin_context = Ops.get_string(
                  NovellLDAP.Settings,
                  "XAD_TREE_ADMIN_CONTEXT",
                  ""
                )
                Builtins.y2milestone(
                  "NovellCifs:XAD_TREE_ADMIN_CONTEXT:%1",
                  xad_tree_admin_context
                )
                if xad_tree_admin_context != nil && xad_tree_admin_context != ""
                  adminDn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
                  adminPassword = Ops.get_string(
                    NovellLDAP.Settings,
                    "XAD_TREE_ADMIN_PASSWORD",
                    ""
                  )
                end
              end
              adminDn = NovellUtils.Nds2Ldap(adminDn)
              Builtins.y2milestone("Cifs:admin:%1", adminDn)
              newcontext = NovellLdapBrowser.LdapBrowser(
                ldap_server,
                ldap_sport,
                true,
                adminDn,
                adminPassword,
                "",
                "(|(ObjectClass=Organization)(ObjectClass=OrganizationalUnit)(ObjectClass=Domain)(ObjectClass=Locality)(ObjectClass=Country))",
                _("Browse for container")
              )
              if newcontext != nil && newcontext != ""
                next if !NovellValidators.ValidateContext(newcontext)
                newcontext = NovellUtils.Ldap2Nds(newcontext)
                if !Builtins.contains(
                    Ops.get_list(
                      NovellCifs.Settings,
                      "CONFIG_CIFS_EDIR_CONTEXTS",
                      []
                    ),
                    newcontext
                  )
                  Ops.set(
                    NovellCifs.Settings,
                    "CONFIG_CIFS_EDIR_CONTEXTS",
                    Builtins.add(
                      Ops.get_list(
                        NovellCifs.Settings,
                        "CONFIG_CIFS_EDIR_CONTEXTS",
                        []
                      ),
                      newcontext
                    )
                  )
                end #add the context to the table
                UI.CloseDialog
                break
              end
            elsif ret2 == :newcontext
              newcontext = Convert.to_string(
                UI.QueryWidget(Id(:newcontext), :Value)
              )
              if newcontext != nil && newcontext != ""
                UI.ChangeWidget(Id(:Ok), :Enabled, true)
                next
              else
                UI.ChangeWidget(Id(:Ok), :Enabled, false)
              end
            elsif ret2 == :Ok
              newcontext = Convert.to_string(
                UI.QueryWidget(Id(:newcontext), :Value)
              )
              if newcontext != nil && newcontext != ""
                if !NovellValidators.ValidateContextExists(
                    newcontext,
                    ldap_server,
                    Builtins.sformat("%1", ldap_sport),
                    EdirUtils.GetLeafNameNDS(
                      Ops.get_string(
                        NovellLDAP.Settings,
                        "CONFIG_LDAP_ADMIN_CONTEXT",
                        ""
                      )
                    ),
                    Ops.get_string(
                      NovellLDAP.Settings,
                      "CONFIG_LDAP_ADMIN_PASSWORD",
                      ""
                    )
                  )
                  Report.Error(
                    _("Context does not exist, enter an existing context")
                  )
                  next
                end

                #       if (!NovellValidators::ValidateContext( newcontext ))
                #            continue;
                newcontext = NovellUtils.Ldap2Nds(newcontext)
                if !Builtins.contains(
                    Ops.get_list(
                      NovellCifs.Settings,
                      "CONFIG_CIFS_EDIR_CONTEXTS",
                      []
                    ),
                    newcontext
                  )
                  Ops.set(
                    NovellCifs.Settings,
                    "CONFIG_CIFS_EDIR_CONTEXTS",
                    Builtins.add(
                      Ops.get_list(
                        NovellCifs.Settings,
                        "CONFIG_CIFS_EDIR_CONTEXTS",
                        []
                      ),
                      newcontext
                    )
                  )
                end #add the context to the table
                UI.CloseDialog
                break
              end
            end
          end
          id_num = 0
          table_items = []
          Builtins.foreach(
            Ops.get_list(NovellCifs.Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
          ) do |item|
            table_items = Builtins.add(table_items, Item(Id(id_num), item))
            id_num = Ops.add(id_num, 1)
          end
          UI.ChangeWidget(Id(:gui_table), :Items, table_items)
          next
        elsif ret == :delBttn
          item = Convert.to_integer(
            UI.QueryWidget(Id(:gui_table), :CurrentItem)
          )
          if Ops.greater_or_equal(item, 0)
            Ops.set(
              NovellCifs.Settings,
              "CONFIG_CIFS_EDIR_CONTEXTS",
              Builtins.remove(
                Ops.get_list(
                  NovellCifs.Settings,
                  "CONFIG_CIFS_EDIR_CONTEXTS",
                  []
                ),
                item
              )
            )
          end
          id_num = 0
          table_items = []
          Builtins.foreach(
            Ops.get_list(NovellCifs.Settings, "CONFIG_CIFS_EDIR_CONTEXTS", [])
          ) do |item2|
            table_items = Builtins.add(table_items, Item(Id(id_num), item2))
            id_num = Ops.add(id_num, 1)
          end
          UI.ChangeWidget(Id(:gui_table), :Items, table_items)
          next
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      deep_copy(ret)
    end


    # Query UI (MultiSelection boxes) for all selected items (password policies)
    #
    # @return [Array<String>] of selected policies
    def GetSelectedItems
      # Generate a list of all selected policies
      selected_policies = Convert.convert(
        UI.QueryWidget(Id(:policies), :SelectedItems),
        :from => "any",
        :to   => "list <integer>"
      )
      selected_policies = [] if selected_policies == nil # avoid nil value
      selected_password_policies = []
      pos = 0
      Builtins.foreach(selected_policies) do |policy|
        selected_password_policies = Builtins.add(
          selected_password_policies,
          Ops.get(@p_policies, policy, "")
        )
      end
      Builtins.y2milestone("selected_policies: %1", selected_password_policies)
      deep_copy(selected_password_policies)
    end

    # Get the password policy data
    def GetPasswordPolicies
      adminData = NovellCifs.getAdminData
      admin = NovellUtils.Nds2Ldap(Ops.get(adminData, 0, ""))
      password = Ops.get(adminData, 1, "")

      use_secure_port = Builtins.tolower(
        Ops.get_string(NovellCifs.Settings, "CONFIG_CIFS_USE_SECURE_PORT", "xx")
      )
      useSsl = "0"
      useSsl = "1" if use_secure_port == "yes"

      ldapServer = ""

      server_list = NovellLDAP.GetRemoteServers(nil)
      if Ops.greater_than(Builtins.size(server_list), 0)
        ldapServer = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      else
        server_list = NovellLDAP.GetLocalServers(nil)
        ldapServer = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
      end

      cifs_ldap_port = Ops.get_integer(
        NovellCifs.Settings,
        "CONFIG_CIFS_LDAP_PORT",
        636
      )

      edir_policies = []
      cmd = Builtins.sformat(
        "%1/getpwpolicies.sh '%2' '%3' %4 %5 %6",
        NovellCifs.cifs_bin_dir,
        admin,
        password,
        ldapServer,
        cifs_ldap_port,
        useSsl
      )

      logcmd = Builtins.sformat(
        "%1/getpwpolicies.sh '%2' '%3' %4 %5 %6",
        NovellCifs.cifs_bin_dir,
        admin,
        "****",
        ldapServer,
        cifs_ldap_port,
        useSsl
      )

      ret = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))

      if NovellCifs.xadTestMode
        Builtins.y2milestone(
          "novellcifs.GetPasswordPolicies.executing:%1, Returned: %2",
          cmd,
          ret
        )
      else
        Builtins.y2milestone(
          "novellcifs.GetPasswordPolicies.executing:%1, Returned: %2",
          logcmd,
          ret
        )
      end

      edirPolicies = Ops.get_string(ret, "stdout", "")

      # REMOVE THIS CODE ****************************************************
      # edirPolicies = "\n\nSample Challenge Set\nSample Password Policy\n";
      # REMOVE THIS CODE ****************************************************

      # get rid of any extra leading or trailing line feeds.
      edirPolicies = NovellUtils.Strip(edirPolicies, "  \t\n")
      if edirPolicies != nil && edirPolicies != ""
        # we have a '\n' separated list of password policies
        edir_policies = Builtins.splitstring(edirPolicies, "\n")
      end

      Builtins.y2milestone(
        "novellcifs.GetPasswordPolicies:edir_policies: %1",
        edir_policies
      )
      deep_copy(edir_policies)
    end

    # CifsConfigure3 dialog
    # @return dialog result
    def CifsConfigure3Dialog
      # CIFS Password Policy options dialog caption
      caption = _("OES CIFS Service Configuration")

      # Read the password policies into the list
      @p_policies = GetPasswordPolicies()

      policy_display_list = []
      if @p_policies != nil && @p_policies != []
        # Generate the widgets for the policies list to display on the UI
        pos = 0
        Builtins.foreach(@p_policies) do |policy|
          element = Item(
            Id(pos), # Select the checkbox if the policy is in the checked_items list
            policy,
            Builtins.contains(
              Ops.get_list(
                NovellCifs.Settings,
                "CONFIG_CIFS_PASSWORD_POLICIES",
                []
              ),
              policy
            )
          )
          policy_display_list = Builtins.add(policy_display_list, element)
          pos = Ops.add(pos, 1)
        end
      end

      # CIFS Password Policy dialog contents
      contents = MarginBox(
        3.5,
        0.2,
        VBox(
          Left(Label(_("Select the Password Policies assigned to CIFS Users:"))),
          MultiSelectionBox(
            Id(:policies),
            Opt(:vstretch),
            "",
            policy_display_list
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.add(
          Ops.get_string(@HELPS, "configure_cifs0", ""),
          Ops.get_string(@HELPS, "configure_cifs3", "")
        ),
        Label.BackButton,
        Label.NextButton
      )

      ret = nil

      if @p_policies == nil || @p_policies == []
        Popup.Message(
          _(
            "There are no password policies currently defined in eDirectory.\n" +
              "To associate policies to users, please re-configure CIFS later\n" +
              "using YaST after adding password policies and users to eDirectory.\n" +
              "CIFS configuration will continue."
          )
        )
        ret = :next
        return deep_copy(ret)
      end

      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          selected = GetSelectedItems()
          # Display popup warning if nothing is selected...
          # if (size(selected) <= 0)
          # 			{
          # 				Popup::Warning(_("Nothing is selected."));
          # 				continue;
          # 			}
          Ops.set(
            NovellCifs.Settings,
            "CONFIG_CIFS_PASSWORD_POLICIES",
            selected
          )
          break
        elsif ret == :allBttn
          set_list = []
          pos = 0
          Builtins.foreach(@p_policies) do |entry|
            set_list = Builtins.add(set_list, pos)
            pos = Ops.add(pos, 1)
          end
          UI.ChangeWidget(Id(:policies), :SelectedItems, set_list)
        elsif ret == :noneBttn
          UI.ChangeWidget(Id(:policies), :SelectedItems, [])
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end

      deep_copy(ret)
    end
  end
end
